#include <sys/utsname.h>
#include <stdlib.h>
#include <strings.h>
#include <hpcxx_rts.h>
#include "temp.h"
#include "MyGlobalPointerPack.h"
#include "RMIRegistry.h"
#include "Component.h"

int main(int argc, char **argv) {

  
  // parse the command line to get
  // serverName, registryHost and PORT
  
  if (argc !=4) {
    printf("\n Please enter the correct command line arguments \n");
    printf("\n TestNewRMI <Server-Name> <Registry-HostName> <Port-Number> \n");
    printf("\n Exiting ... \n");
    exit(EXIT_FAILURE);
  }

  char *serverName = strdup(argv[1]);
  char *registryHost = strdup(argv[2]);
  unsigned short PORT = atoi(argv[3]);
  
  printf("Server: serverName: \t %s\n",serverName);
  printf("Server: registryHost: \t %s\n",registryHost);
  printf("Server: registryPort: \t %d\n",PORT);
  
  //init hpcxx
  fflush(stdout);
  HPCxx_Group *group;
  hpcxx_init(argc, argv,group);
  fflush(stdout); 

  // create a global pointer to the "Component" object
  printf("Server: creating global pointer to \"Component\" object\n");
  fflush(stdout);
  Component *componentLocal = new Component();
  Component compLocal;
  HPCxx_GlobalPtr<Component> gpComponent(componentLocal);
  printf("Server: global pointer to \"Component\" object created\n");
  fflush(stdout);

  //Get the startpoint of the RMIregistry
  printf("Server: calling function attach_to_registry \n");
  fflush(stdout);
  RMIRegistry<Component> *rmiRegistry = new RMIRegistry<Component>();
  rmiRegistry->attachToRegistry(registryHost, PORT);
  printf("Server: in main(): attach_to_registry done\n");
  fflush(stdout);

  // register local object "Component" in registry
  // rebind is used instead of bind as if a server is already bound with this name 
  // then the current server will override it. Note that the registry is 
  // exepected to run forever

  printf("Server: calling rebind_registry() to register ..\n");
  printf("Server: \"Component\" at rmi://%s:%hu/%s\n", registryHost, PORT, serverName);
  fflush(stdout);
  char *service = (char *)malloc(sizeof(char)*100);
  sprintf(service,"rmi://%s:%hu/%s", registryHost, PORT, serverName);
  //gpComponent.setFullyQualifiedFileName("rmitest.ServerImpl");
  HPCxx_String rmiURL(service);
  cout << "Server: Registry service requested at " << rmiURL << endl; 
  HPCxx_String FQN("rmitest.ServerImpl");
  rmiRegistry->rebindRegistry(gpComponent, service, FQN);

  free(serverName);
  free(registryHost);

  // waiting to be killed
  HPCxx_Sync<int> killMe;
  int waitToBeKilled = killMe;
}





